//======================================================================
// LView - ListView control demonstration
//
// Written for the book Programming Windows CE
// Copyright (C) 1998 Douglas Boling
//======================================================================
#include <windows.h>                 // For all that Windows stuff
#include <commctrl.h>                // Command bar includes
#include "LView.h"                   // Program-specific stuff
//----------------------------------------------------------------------
// Global data
//
const TCHAR szAppName[] = TEXT ("LView");
HINSTANCE hInst;                     // Program instance handle
HWND hMain;

//
// Data for simulated database
//
#define LVCNT 2000
LVDATAITEM lvdatabase[LVCNT];

// Defines and data for list view control cache
#define  CACHESIZE       100
#define  TOPCACHESIZE    100
#define  BOTCACHESIZE    100

INT nCacheItemStart = 0, nCacheSize = 0;
LVDATAITEM lvdiCache[CACHESIZE];
LVDATAITEM lvdiTopCache[TOPCACHESIZE];
LVDATAITEM lvdiBotCache[BOTCACHESIZE];

// Message dispatch table for MainWindowProc
const struct decodeUINT MainMessages[] = {
    WM_CREATE, DoCreateMain,
    WM_SIZE, DoSizeMain,
    WM_NOTIFY, DoNotifyMain,
    WM_COMMAND, DoCommandMain,
    WM_DESTROY, DoDestroyMain,
};
// Command message dispatch for MainWindowProc
const struct decodeCMD MainCommandItems[] = {
    IDM_EXIT, DoMainCommandExit,
    IDC_LICON, DoMainCommandChView,
    IDC_SICON, DoMainCommandChView,
    IDC_LIST, DoMainCommandChView,
    IDC_RPT, DoMainCommandChView,
    IDM_ABOUT, DoMainCommandAbout,
};
// Standard file bar button structure
const TBBUTTON tbCBCmboBtns[] = {
//  BitmapIndex       Command    State        Style        UserData String
    {0,               0,         0,           TBSTYLE_SEP,        0,  0},
    {VIEW_LARGEICONS, IDC_LICON, TBSTATE_ENABLED,
                                              TBSTYLE_CHECKGROUP, 0,  0},
    {VIEW_SMALLICONS, IDC_SICON, TBSTATE_ENABLED,
                                              TBSTYLE_CHECKGROUP, 0,  0},
    {VIEW_LIST,       IDC_LIST,  TBSTATE_ENABLED,
                                              TBSTYLE_CHECKGROUP, 0,  0},
    {VIEW_DETAILS,    IDC_RPT,   TBSTATE_ENABLED | TBSTATE_CHECKED,
                                              TBSTYLE_CHECKGROUP, 0,  0}
};
//======================================================================
// Program entry point
//
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPWSTR lpCmdLine, int nCmdShow) {
    MSG msg;
    HWND hwndMain;
    int rc = 0;

    // Initialize application.
    rc = InitApp (hInstance);
    if (rc) return rc;

    // Initialize this instance.
    hwndMain = InitInstance (hInstance, lpCmdLine, nCmdShow);
    if (hwndMain == 0)
        return 0x10;

    hMain = hwndMain;
    // Application message loop
    while (GetMessage (&msg, NULL, 0, 0)) {
        TranslateMessage (&msg);
        DispatchMessage (&msg);
    }
    // Instance cleanup
    return TermInstance (hInstance, msg.wParam);
}
//----------------------------------------------------------------------
// InitApp - Application initialization
//
int InitApp (HINSTANCE hInstance) {
    WNDCLASS     wc;
    INITCOMMONCONTROLSEX icex;

    // Register application main window class.
    wc.style = 0;                             // Window style
    wc.lpfnWndProc = MainWndProc;             // Callback function
    wc.cbClsExtra = 0;                        // Extra class data
    wc.cbWndExtra = 0;                        // Extra window data
    wc.hInstance = hInstance;                 // Owner handle
    wc.hIcon = NULL,                          // Application icon
    wc.hCursor = NULL;                        // Default cursor
    wc.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH);
    wc.lpszMenuName =  NULL;                  // Menu name
    wc.lpszClassName = szAppName;             // Window class name

    if (RegisterClass (&wc) == 0) return 1;

    // Load the command bar common control class.
    icex.dwSize = sizeof (INITCOMMONCONTROLSEX);
    icex.dwICC = ICC_LISTVIEW_CLASSES;
    InitCommonControlsEx (&icex);


    // Initialize the fictional database.
    InitDatabase ();
    return 0;
}
//----------------------------------------------------------------------
// InitInstance - Instance initialization
//
HWND InitInstance (HINSTANCE hInstance, LPWSTR lpCmdLine, int nCmdShow){
    HWND hWnd;

    // Save program instance handle in global variable.
    hInst = hInstance;

    // Create main window.
    hWnd = CreateWindow (szAppName,            // Window class
                         TEXT ("LView"),       // Window title
                         WS_VISIBLE,           // Style flags
                         CW_USEDEFAULT,        // x position
                         CW_USEDEFAULT,        // y position
                         CW_USEDEFAULT,        // Initial width
                         CW_USEDEFAULT,        // Initial height
                         NULL,                 // Parent
                         NULL,                 // Menu, must be null
                         hInstance,            // Application instance
                         NULL);                // Pointer to create
                                               // parameters
    // Return fail code if window not created.
    if (!IsWindow (hWnd)) return 0;

    // Standard show and update calls
    ShowWindow (hWnd, nCmdShow);
    UpdateWindow (hWnd);
    return hWnd;
}
//----------------------------------------------------------------------
// TermInstance - Program cleanup
//
int TermInstance (HINSTANCE hInstance, int nDefRC) {

    // Flush caches used with list view control.
    FlushMainCache ();
    FlushEndCaches ();
    return nDefRC;
}
//======================================================================
// Message-handling procedures for MainWindow
//----------------------------------------------------------------------
// MainWndProc - Callback function for application window
//
LRESULT CALLBACK MainWndProc(HWND hWnd, UINT wMsg, WPARAM wParam,
                             LPARAM lParam) {
    INT i;
    //
    // Search message list to see if we need to handle this
    // message.  If in list, call procedure.
    //
    for (i = 0; i < dim(MainMessages); i++) {
        if (wMsg == MainMessages[i].Code)
            return (*MainMessages[i].Fxn)(hWnd, wMsg, wParam, lParam);
    }
    return DefWindowProc(hWnd, wMsg, wParam, lParam);
}
//----------------------------------------------------------------------
// DoCreateMain - Process WM_CREATE message for window.
//
LRESULT DoCreateMain (HWND hWnd, UINT wMsg, WPARAM wParam,
                      LPARAM lParam) {
    HWND hwndCB, hwndLV;
    INT i, nHeight;
    LPCREATESTRUCT lpcs;
    HIMAGELIST himlLarge, himlSmall;
    HICON hIcon;

    // Convert lParam into pointer to create structure.
    lpcs = (LPCREATESTRUCT) lParam;

    // Create a command bar.
    hwndCB = CommandBar_Create (hInst, hWnd, IDC_CMDBAR);

    // Insert a menu.
    CommandBar_InsertMenubar (hwndCB, hInst, ID_MENU, 0);

    // Add bitmap list followed by buttons.
    CommandBar_AddBitmap (hwndCB, HINST_COMMCTRL, IDB_VIEW_SMALL_COLOR,
                          VIEW_BMPS, 0, 0);
    CommandBar_AddButtons (hwndCB, dim(tbCBCmboBtns), tbCBCmboBtns);

    // Add exit button to command bar.
    CommandBar_AddAdornments (hwndCB, 0, 0);
    nHeight = CommandBar_Height (hwndCB);
    //
    // Create the list view control.
    //
    hwndLV = CreateWindowEx (0, WC_LISTVIEW, TEXT (""),
                             LVS_REPORT | LVS_SINGLESEL |
                             LVS_OWNERDATA | WS_VISIBLE | WS_CHILD |
                             WS_VSCROLL, 0, nHeight, lpcs->cx,
                             lpcs->cy - nHeight, hWnd,
                             (HMENU)IDC_LISTVIEW,
                             lpcs->hInstance, NULL);
    // Destroy frame if window not created.
    if (!IsWindow (hwndLV)) {
        DestroyWindow (hWnd);
        return 0;
    }
    // Add columns.
    {
        LVCOLUMN lvc;

        lvc.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM;
        lvc.fmt = LVCFMT_LEFT;
        lvc.cx = 150;
        lvc.pszText = TEXT ("Name");
        lvc.iSubItem = 0;
        SendMessage (hwndLV, LVM_INSERTCOLUMN, 0, (LPARAM)&lvc);

        lvc.mask |= LVCF_SUBITEM;
        lvc.pszText = TEXT ("Type");
        lvc.cx = 100;
        lvc.iSubItem = 1;
        SendMessage (hwndLV, LVM_INSERTCOLUMN, 1, (LPARAM)&lvc);

        lvc.mask |= LVCF_SUBITEM;
        lvc.pszText = TEXT ("Size");
        lvc.cx = 100;
        lvc.iSubItem = 2;
        SendMessage (hwndLV, LVM_INSERTCOLUMN, 2, (LPARAM)&lvc);
    }
    // Add items.
    ListView_SetItemCount (hwndLV, LVCNT);
    LoadTopCache ();
    LoadBotCache ();

    // Create image list control for bitmaps for minimized bands.
    i = GetSystemMetrics (SM_CXICON);
    himlLarge = ImageList_Create(i, i, ILC_COLOR, 2, 0);
    i = GetSystemMetrics (SM_CXSMICON);
    himlSmall = ImageList_Create(i, i, ILC_COLOR, 2, 0);

    // Load large and small icons into their respective image lists.
    hIcon = LoadIcon (hInst, TEXT ("DocIcon"));
    i = ImageList_AddIcon (himlLarge, hIcon);

    hIcon = LoadImage (hInst, TEXT ("DocIcon"), IMAGE_ICON, 16, 16,
                       LR_DEFAULTCOLOR);
    ImageList_AddIcon (himlSmall, hIcon);

    ListView_SetImageList (hwndLV, himlLarge, LVSIL_NORMAL);
    ListView_SetImageList (hwndLV, himlSmall, LVSIL_SMALL);

    // Set cool new styles.
    ListView_SetExtendedListViewStyle (hwndLV, LVS_EX_GRIDLINES |
                                       LVS_EX_HEADERDRAGDROP |
                                       LVS_EX_FULLROWSELECT);
    return 0;
}
//----------------------------------------------------------------------
// DoSizeMain - Process WM_SIZE message for window.
//
LRESULT DoSizeMain (HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam){
    HWND hwndLV;
    RECT rect;

    hwndLV = GetDlgItem (hWnd, IDC_LISTVIEW);

    // Adjust the size of the client rect to take into account
    // the command bar height.
    GetClientRect (hWnd, &rect);
    rect.top += CommandBar_Height (GetDlgItem (hWnd, IDC_CMDBAR));

    SetWindowPos (hwndLV, NULL, rect.left, rect.top,
                  rect.right - rect.left, rect.bottom - rect.top,
                  SWP_NOZORDER);
    return 0;
}
//----------------------------------------------------------------------
// DoNotifyMain - Process WM_NOTIFY message for window.
//
LRESULT DoNotifyMain (HWND hWnd, UINT wMsg, WPARAM wParam,
                      LPARAM lParam) {
    int idItem;
    LPNMHDR    pnmh;
    LPNMLISTVIEW pnmlv;
    NMLVDISPINFO *pLVdi;
    PLVDATAITEM pdi;     // Pointer to data
    LPNMLVCACHEHINT pLVch;
    HWND hwndLV;

    // Parse the parameters.
    idItem = (int) wParam;
    pnmh = (LPNMHDR)lParam;
    hwndLV = pnmh->hwndFrom;

    if (idItem == IDC_LISTVIEW) {
        pnmlv = (LPNMLISTVIEW)lParam;

        switch (pnmh->code) {
        case LVN_GETDISPINFO:
            pLVdi = (NMLVDISPINFO *)lParam;

            // Get a pointer to the data either from the cache
            // or from the actual database.
            pdi = GetItemData (pLVdi->item.iItem);

            if (pLVdi->item.mask & LVIF_IMAGE)
                pLVdi->item.iImage = pdi->nImage;

            if (pLVdi->item.mask & LVIF_PARAM)
                pLVdi->item.lParam = 0;

            if (pLVdi->item.mask & LVIF_STATE)
                pLVdi->item.state = pdi->nState;

            if (pLVdi->item.mask & LVIF_TEXT) {
                switch (pLVdi->item.iSubItem) {
                case 0:
                    lstrcpy (pLVdi->item.pszText, pdi->szName);
                    break;
                case 1:
                    lstrcpy (pLVdi->item.pszText, pdi->szType);
                    break;
                case 2:
                    wsprintf (pLVdi->item.pszText, TEXT ("%d"),
                              pdi->nSize);
                    break;
                }
            }
            break;

        case LVN_ODCACHEHINT:
            pLVch = (LPNMLVCACHEHINT)lParam;
            LoadMainCache (pLVch->iFrom, pLVch->iTo);
            break;

        case LVN_ODFINDITEM:
            // We should do a reverse look up here to see if
            // an item exists for the text passed.
            return -1;
        }
    }
    return 0;
}
//----------------------------------------------------------------------
// DoCommandMain - Process WM_COMMAND message for window.
//
LRESULT DoCommandMain (HWND hWnd, UINT wMsg, WPARAM wParam,
                       LPARAM lParam) {
    WORD idItem, wNotifyCode;
    HWND hwndCtl;
    INT  i;

    // Parse the parameters.
    idItem = (WORD) LOWORD (wParam);
    wNotifyCode = (WORD) HIWORD (wParam);
    hwndCtl = (HWND) lParam;

    // Call routine to handle control message.
    for (i = 0; i < dim(MainCommandItems); i++) {
        if (idItem == MainCommandItems[i].Code)
            return (*MainCommandItems[i].Fxn)(hWnd, idItem, hwndCtl,
                                              wNotifyCode);
    }
    return 0;
}
//----------------------------------------------------------------------
// DoDestroyMain - Process WM_DESTROY message for window.
//
LRESULT DoDestroyMain (HWND hWnd, UINT wMsg, WPARAM wParam,
                       LPARAM lParam) {
    PostQuitMessage (0);
    return 0;
}
//======================================================================
// Command handler routines
//----------------------------------------------------------------------
// DoMainCommandExit - Process Program Exit command.
//
LPARAM DoMainCommandExit (HWND hWnd, WORD idItem, HWND hwndCtl,
                          WORD wNotifyCode) {

    SendMessage (hWnd, WM_CLOSE, 0, 0);
    return 0;
}
//----------------------------------------------------------------------
// DoMainCommandChView - Process View xxx command.
//
LPARAM DoMainCommandChView (HWND hWnd, WORD idItem, HWND hwndCtl,
                            WORD wNotifyCode) {
    HWND hwndLV;
    LONG lStyle;

    hwndLV = GetDlgItem (hWnd, IDC_LISTVIEW);

    lStyle = GetWindowLong (hwndLV, GWL_STYLE);
    lStyle &= ~LVS_TYPEMASK;

    switch (idItem) {
    case IDC_LICON:
        lStyle |= LVS_ICON;
        break;
    case IDC_SICON:
        lStyle |= LVS_SMALLICON;
        break;
    case IDC_LIST:
        lStyle |= LVS_LIST;
        break;
    case IDC_RPT:
        lStyle |= LVS_REPORT;
        break;
    }
    SetWindowLong (hwndLV, GWL_STYLE, lStyle);
    return 0;
}
//----------------------------------------------------------------------
// DoMainCommandAbout - Process the Help | About menu command.
//
LPARAM DoMainCommandAbout(HWND hWnd, WORD idItem, HWND hwndCtl,
                          WORD wNotifyCode) {

    // Use DialogBox to create modal dialog box.
    DialogBox (hInst, TEXT ("aboutbox"), hWnd, AboutDlgProc);
    return 0;
}
//======================================================================
// About Dialog procedure
//
BOOL CALLBACK AboutDlgProc(HWND hWnd, UINT wMsg, WPARAM wParam,
                          LPARAM lParam) {

    switch (wMsg) {
        case WM_COMMAND:
            switch (LOWORD (wParam)) {
                case IDOK:
                case IDCANCEL:
                    EndDialog (hWnd, 0);
                    return TRUE;
            }
        break;
    }
    return FALSE;
}
//======================================================================
// Helper routines for list view control management
//----------------------------------------------------------------------
// AddItem - Add an item to the list view control.
//
INT AddItem (HWND hwndCtl, INT nItem, LPTSTR pszName, LPTSTR pszType,
             INT nSize) {
    LVITEM lvi;
    TCHAR szTmp[40];

    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;
    lvi.iItem = nItem;
    lvi.iSubItem = 0;
    lvi.pszText = pszName;
    lvi.iImage = 0;
    lvi.lParam = nItem;
    SendMessage (hwndCtl, LVM_INSERTITEM, 0, (LPARAM)&lvi);

    lvi.mask = LVIF_TEXT;
    lvi.iItem = nItem;
    lvi.iSubItem = 1;
    lvi.pszText = pszType;
    SendMessage (hwndCtl, LVM_SETITEM, 0, (LPARAM)&lvi);

    wsprintf (szTmp, TEXT ("%d"), nSize);
    lvi.mask = LVIF_TEXT;
    lvi.iItem = nItem;
    lvi.iSubItem = 2;
    lvi.pszText = szTmp;
    SendMessage (hwndCtl, LVM_SETITEM, 0, (LPARAM)&lvi);

    return 0;
}
//----------------------------------------------------------------------
// GetItemData - This routine returns a pointer to the data.  It
// first checks the caches before calling directly to the database.
//
PLVDATAITEM GetItemData (INT nItem) {
    INT nCacheIndex;
    PLVDATAITEM pdi;

    // See if it's in the top cache.
    if (nItem < TOPCACHESIZE) {

        nCacheIndex = nItem;
        pdi = &lvdiTopCache[nCacheIndex];
    }

    // See if it's in the bottom cache.
    else if (nItem > LVCNT - BOTCACHESIZE) {

        nCacheIndex = nItem - (LVCNT - BOTCACHESIZE);
        pdi = &lvdiBotCache[nCacheIndex];
    }
    // See if item's in the main cache.
    else if ((nItem >= nCacheItemStart) &&
             (nItem < nCacheItemStart + nCacheSize)) {

        nCacheIndex = nItem - nCacheItemStart;
        pdi = &lvdiCache[nCacheIndex];
    }

    // Otherwise it's not in any cache.
    else
        pdi = GetDatabaseItem (nItem);

    return pdi;
}
//----------------------------------------------------------------------
INT LoadACache (PLVDATAITEM pCache, INT nStart, INT nSize) {
    PLVDATAITEM pdi;
    INT i;

    for (i = 0; i < nSize; i++) {
        // Get a pointer to the data.
        pdi = GetDatabaseItem (nStart+i);

        // Save the data in the cache.
        lstrcpy (pCache[i].szName, pdi->szName);
        lstrcpy (pCache[i].szType, pdi->szType);
        pCache[i].nSize = pdi->nSize;
        pCache[i].nImage = pdi->nImage;
        pCache[i].nState = pdi->nState;
    }
    return 0;
}
//----------------------------------------------------------------------
// LoadMainCache - This routine loads the hint cache.  If the
// recommended range is already in the top or bottom caches, the range
// is adjusted to grab items outside the end caches.
//
// The logic expects the total number of items to be greater than the
// size of the start and end caches.
//
INT LoadMainCache (INT nStart, INT nEnd) {
    INT nOverlap;

    // Size the hint range to fit the cache.
    if (nEnd - nStart > CACHESIZE)
        nEnd = nStart + CACHESIZE;

    // See if end of hint in bottom cache.
    if (nEnd > LVCNT - BOTCACHESIZE) {

        // If completely in bottom cache, keep old data.
        if (nStart > LVCNT - BOTCACHESIZE)
            return 0;

        // If partial overlap, adjust end points to get data just
        // above the bottom cache.
        nOverlap = nEnd - (LVCNT - BOTCACHESIZE);
        nEnd = LVCNT - BOTCACHESIZE - 1;
        if (nStart - nOverlap < TOPCACHESIZE)
            nStart = TOPCACHESIZE;
        else
            nStart -= nOverlap;
    }
    // See if start of hint in top cache.
    if (nStart < TOPCACHESIZE) {

        // If completely in top cache, keep old data.
        if (nEnd < TOPCACHESIZE)
            return 0;

        // Adjust the starting value to just beyond top cache end.
        nOverlap = TOPCACHESIZE - nStart;
        nStart = TOPCACHESIZE;
        if (nOverlap + nEnd > (LVCNT - BOTCACHESIZE))
            nEnd = LVCNT - BOTCACHESIZE;
        else
            nEnd += nOverlap;
    }
    // If hint already completely contained in the cache, exit.
    if ((nStart >= nCacheItemStart) && 
        (nEnd < nCacheItemStart + nCacheSize))
        return 0;

    // Flush old data in cache.  We should really be smart here to
    // see whether part of the data is already in the cache.
    FlushMainCache ();

    // Load the new data.
    nCacheSize = nEnd - nStart;
    nCacheItemStart = nStart;
    LoadACache (lvdiCache, nStart, nCacheSize);
    return 0;
}
//----------------------------------------------------------------------
INT LoadTopCache (void) {

    LoadACache (lvdiTopCache, 0, TOPCACHESIZE);
    return 0;
}
//----------------------------------------------------------------------
INT LoadBotCache (void) {

    LoadACache (lvdiBotCache, LVCNT - BOTCACHESIZE, BOTCACHESIZE);
    return 0;
}
//----------------------------------------------------------------------
void FlushMainCache (void) {
    INT i;

    // Send the data back to the database.
    for (i = 0; i < nCacheSize; i++) {
        SetDatabaseItem (nCacheItemStart+i, &lvdiCache[i]);
    }
    return;
}
//----------------------------------------------------------------------
void FlushEndCaches (void) {
    INT i;

    // Flush the top cache.
    for (i = 0; i < TOPCACHESIZE; i++) {
        SetDatabaseItem (i, &lvdiCache[i]);
    }
    // Flush the bottom cache.
    for (i = 0; i < BOTCACHESIZE; i++) {
        SetDatabaseItem (LVCNT - BOTCACHESIZE + i, &lvdiBotCache[i]);
    }
    return;
}
//======================================================================
// Code for fictional database to be displayed in the list view control
//
//----------------------------------------------------------------------
// InitDatabaseItem - Copy an item into the database.
//
INT InitDatabaseItem (INT nItem, LPTSTR pszName, LPTSTR pszType,
                      INT nSize) {

    lstrcpy (lvdatabase[nItem].szName, pszName);
    lstrcpy (lvdatabase[nItem].szType, pszType);
    lvdatabase[nItem].nSize = nSize;
    lvdatabase[nItem].nImage = 0;
    lvdatabase[nItem].nState = 0;
    return 0;
}
//----------------------------------------------------------------------
// InitDatabase - Create fictional data for fictional database.
//
void InitDatabase (void) {
    TCHAR szName[64];
    TCHAR szType[64];
    HCURSOR hOldCur;
    INT i;

    hOldCur = SetCursor (LoadCursor (NULL, IDC_WAIT));

    for (i = 0; i < LVCNT; i++) {
        wsprintf (szName, TEXT ("File%d"), i);
        wsprintf (szType, TEXT ("Type%d"), 1000 - i);
        InitDatabaseItem (i, szName, szType, i+1000);
    }
    SetCursor (hOldCur);
    return;
}
//----------------------------------------------------------------------
// GetDatabaseItem - Return a pointer to data in the database.
//
PLVDATAITEM GetDatabaseItem (INT nItem) {

    // Normally, this would be more work. But since
    // we have only a simulated data store, the
    // code is trivial.
    return &lvdatabase[nItem];
}
//----------------------------------------------------------------------
// SetDatabaseItem - Copy data from list view control back into database.
//
INT SetDatabaseItem (INT nItem, PLVDATAITEM pIn) {

    lstrcpy (lvdatabase[nItem].szName, pIn->szName);
    lstrcpy (lvdatabase[nItem].szType, pIn->szType);
    lvdatabase[nItem].nSize = pIn->nSize;
    lvdatabase[nItem].nImage = pIn->nImage;
    lvdatabase[nItem].nState = pIn->nState;
    return 0;
}
