//======================================================================
// AlbumDB - A Windows CE database
//
// Written for the book Programming Windows CE
// Copyright (C) 1998 Douglas Boling
//======================================================================
#include <windows.h>                 // For all that Windows stuff
#include <windowsx.h>                // For Window Controls macros
#include <commctrl.h>                // Command bar includes

#include "AlbumDB.h"                 // Program-specific stuff

//----------------------------------------------------------------------
// Global data
//
const TCHAR szAppName[] = TEXT ("AlbumDB");
HINSTANCE hInst;                     // Program instance handle
HANDLE g_hDB = 0;                    // Handle to album database
CEOID g_oidDB = 0;                   // Handle to album database
INT g_nLastSort = PID_NAME;          // Last sort order used

// These two variables represent a one item cache for
// the list view control.
int g_nLastItem = -1;
LPBYTE g_pLastRecord = 0;

// Message dispatch table for MainWindowProc
const struct decodeUINT MainMessages[] = {
    WM_CREATE, DoCreateMain,
    WM_SIZE, DoSizeMain,
    WM_COMMAND, DoCommandMain,
    WM_NOTIFY, DoNotifyMain,
    WM_DESTROY, DoDestroyMain,
    DB_CEOID_CHANGED, DoDbNotifyMain,
    DB_CEOID_CREATED, DoDbNotifyMain,
    DB_CEOID_RECORD_DELETED, DoDbNotifyMain,
};

// Command message dispatch for MainWindowProc
const struct decodeCMD MainCommandItems[] = {
    IDM_DELDB, DoMainCommandDelDB,
    IDM_EXIT, DoMainCommandExit,
    IDM_NEW, DoMainCommandNew,
    IDM_EDIT, DoMainCommandEdit,
    IDM_DELETE, DoMainCommandDelete,
    IDM_SORTNAME, DoMainCommandSort,
    IDM_SORTARTIST, DoMainCommandSort,
    IDM_SORTCATEGORY, DoMainCommandSort,
    IDM_ABOUT, DoMainCommandAbout,
};
// Album category strings; must be alphabetical.
const TCHAR *pszCategories[] = {TEXT ("Classical"), TEXT ("Country"), 
                                TEXT ("New Age"), TEXT ("Rock")};
//======================================================================
// Program entry point
//
int WINAPI WinMain (HINSTANCE hInstance, HINSTANCE hPrevInstance,
                    LPWSTR lpCmdLine, int nCmdShow) {
    HWND hwndMain;
    MSG msg;
    int rc = 0;

    // Initialize application.
    rc = InitApp (hInstance);
    if (rc) return rc;

    // Initialize this instance.
    hwndMain = InitInstance (hInstance, lpCmdLine, nCmdShow);
    if (hwndMain == 0)
        return 0x10;

    // Application message loop
    while (GetMessage (&msg, NULL, 0, 0)) {
        TranslateMessage (&msg);
        DispatchMessage (&msg);
    }
    // Instance cleanup
    return TermInstance (hInstance, msg.wParam);
}
//----------------------------------------------------------------------
// InitApp - Application initialization
//
int InitApp (HINSTANCE hInstance) {
    WNDCLASS wc;
    INITCOMMONCONTROLSEX icex;

    // Register application main window class.
    wc.style = 0;                             // Window style
    wc.lpfnWndProc = MainWndProc;             // Callback function
    wc.cbClsExtra = 0;                        // Extra class data
    wc.cbWndExtra = 0;                        // Extra window data
    wc.hInstance = hInstance;                 // Owner handle
    wc.hIcon = NULL,                          // Application icon
    wc.hCursor = NULL;                        // Default cursor
    wc.hbrBackground = (HBRUSH) GetStockObject (WHITE_BRUSH);
    wc.lpszMenuName =  NULL;                  // Menu name
    wc.lpszClassName = szAppName;             // Window class name

    if (RegisterClass (&wc) == 0) return 1;

    // Load the command bar common control class.
    icex.dwSize = sizeof (INITCOMMONCONTROLSEX);
    icex.dwICC = ICC_BAR_CLASSES | ICC_TREEVIEW_CLASSES | 
                 ICC_LISTVIEW_CLASSES;
    InitCommonControlsEx (&icex);
    return 0;
}
//----------------------------------------------------------------------
// InitInstance - Instance initialization
// 
HWND InitInstance (HINSTANCE hInstance, LPWSTR lpCmdLine, int nCmdShow){
    HWND hWnd;

    // Save program instance handle in global variable.
    hInst = hInstance;

    // Create main window.
    hWnd = CreateWindow (szAppName, TEXT ("AlbumDB"), WS_VISIBLE, 
                         CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,   
                         CW_USEDEFAULT, NULL, NULL, hInstance, NULL);           
                                          
    // Return fail code if window not created.
    if (!IsWindow (hWnd)) return 0;

    // Standard show and update calls
    ShowWindow (hWnd, nCmdShow);
    UpdateWindow (hWnd);
    return hWnd;
}
//----------------------------------------------------------------------
// TermInstance - Program cleanup
//
int TermInstance (HINSTANCE hInstance, int nDefRC) {
    // Close the opened database.
    if (g_hDB)
        CloseHandle (g_hDB);
    // Free the last db query if saved.
    ClearCache ();

    return nDefRC;
}
//======================================================================
// Message handling procedures for MainWindow
//----------------------------------------------------------------------
// MainWndProc - Callback function for application window
//
LRESULT CALLBACK MainWndProc (HWND hWnd, UINT wMsg, WPARAM wParam, 
                              LPARAM lParam) {
    INT i;
    //
    // Search message list to see if we need to handle this
    // message.  If in list, call procedure.
    //
    for (i = 0; i < dim(MainMessages); i++) {
        if (wMsg == MainMessages[i].Code)
            return (*MainMessages[i].Fxn)(hWnd, wMsg, wParam, lParam);
    }
    return DefWindowProc (hWnd, wMsg, wParam, lParam);
}
//----------------------------------------------------------------------
// DoCreateMain - Process WM_CREATE message for window.
//
LRESULT DoCreateMain (HWND hWnd, UINT wMsg, WPARAM wParam, 
                      LPARAM lParam) {
    HWND hwndCB, hwndChild;
    INT  nHeight, nCnt;
    RECT rect;
    LPCREATESTRUCT lpcs;

    // Convert lParam into pointer to create structure.
    lpcs = (LPCREATESTRUCT) lParam;

    // Create a minimal command bar that only has a menu and an 
    // exit button.
    hwndCB = CommandBar_Create (hInst, hWnd, IDC_CMDBAR);
    // Insert the menu.
    CommandBar_InsertMenubar (hwndCB, hInst, ID_MENU, 0);
    // Add exit button to command bar. 
    CommandBar_AddAdornments (hwndCB, 0, 0);
    nHeight = CommandBar_Height (hwndCB);

    // Open the album database.  If one doesn't exist, create it.
    g_hDB = OpenCreateDB (hWnd, &nCnt);
    if (g_hDB == INVALID_HANDLE_VALUE) {
        MessageBox (hWnd, TEXT ("Could not open database."), szAppName,
                    MB_OK);
        DestroyWindow (hWnd);
        return 0;
    }
    // Create the list view control in right pane.
    SetRect (&rect, 0, nHeight, lpcs->cx, lpcs->cy - nHeight);
    hwndChild = CreateLV (hWnd, &rect);

    // Destroy frame if window not created.
    if (!IsWindow (hwndChild)) {
        DestroyWindow (hWnd);
        return 0;
    }
    ListView_SetItemCount (hwndChild, nCnt);
    return 0;
}
//----------------------------------------------------------------------
// DoSizeMain - Process WM_SIZE message for window.
// 
LRESULT DoSizeMain (HWND hWnd, UINT wMsg, WPARAM wParam, LPARAM lParam){
    HWND hwndLV;
    RECT rect;

    hwndLV = GetDlgItem (hWnd, ID_LISTV);

    // Adjust the size of the client rect to take into account
    // the command bar height.
    GetClientRect (hWnd, &rect);
    rect.top += CommandBar_Height (GetDlgItem (hWnd, IDC_CMDBAR));

    SetWindowPos (hwndLV, NULL, rect.left, rect.top, 
                  (rect.right - rect.left), rect.bottom - rect.top,
                  SWP_NOZORDER);
    return 0;
}
//----------------------------------------------------------------------
// DoCommandMain - Process WM_COMMAND message for window.
// 
LRESULT DoCommandMain (HWND hWnd, UINT wMsg, WPARAM wParam, 
                       LPARAM lParam) {
    WORD idItem, wNotifyCode;
    HWND hwndCtl;
    INT  i;

    // Parse the parameters.
    idItem = (WORD) LOWORD (wParam);
    wNotifyCode = (WORD) HIWORD (wParam);
    hwndCtl = (HWND) lParam;

    // Call routine to handle control message.
    for (i = 0; i < dim(MainCommandItems); i++) {
        if (idItem == MainCommandItems[i].Code)
            return (*MainCommandItems[i].Fxn)(hWnd, idItem, hwndCtl, 
                                              wNotifyCode);
    }
    return 0;
}
//----------------------------------------------------------------------
// DoNotifyMain - Process DB_CEOID_xxx messages for window.
// 
LRESULT DoDbNotifyMain (HWND hWnd, UINT wMsg, WPARAM wParam, 
                        LPARAM lParam) {
    switch (wMsg) {
    case DB_CEOID_CHANGED:
        InvalidateRect (GetDlgItem (hWnd, ID_LISTV), NULL, TRUE);
        break;
    case DB_CEOID_CREATED:
        ReopenDatabase (hWnd, -1);
        break;
    case DB_CEOID_RECORD_DELETED:
        ReopenDatabase (hWnd, -1);
        break;
    }
    return 0;
}
//----------------------------------------------------------------------
// DoNotifyMain - Process WM_NOTIFY message for window.
// 
LRESULT DoNotifyMain (HWND hWnd, UINT wMsg, WPARAM wParam, 
                      LPARAM lParam) {
    int idItem, i;
    LPNMHDR pnmh;
    LPNMLISTVIEW pnmlv;
    NMLVDISPINFO *pLVdi;
    LVCACHEDATA data;
    HWND hwndLV;

    // Parse the parameters.
    idItem = (int) wParam;
    pnmh = (LPNMHDR)lParam;
    hwndLV = pnmh->hwndFrom;

    if (idItem == ID_LISTV) {
        pnmlv = (LPNMLISTVIEW)lParam;

        switch (pnmh->code) {
        case LVN_GETDISPINFO:
            pLVdi = (NMLVDISPINFO *)lParam;

            // Get a pointer to the data either from the cache
            // or from the actual database.
            GetItemData (pLVdi->item.iItem, &data);

            if (pLVdi->item.mask & LVIF_IMAGE) 
                pLVdi->item.iImage = 0;

            if (pLVdi->item.mask & LVIF_PARAM) 
                pLVdi->item.lParam = 0;

            if (pLVdi->item.mask & LVIF_STATE) 
                pLVdi->item.state = 0;

            if (pLVdi->item.mask & LVIF_TEXT) {
                switch (pLVdi->item.iSubItem) {
                case 0:
                    lstrcpy (pLVdi->item.pszText, data.Album.szName);
                    break;
                case 1:
                    lstrcpy (pLVdi->item.pszText, data.Album.szArtist);
                    break;
                case 2:
                    lstrcpy (pLVdi->item.pszText, 
                             pszCategories[data.Album.sCategory]);
                    break;
                }
            }
            break;

        // Ignore cache hinting for db example.
        case LVN_COLUMNCLICK:
            i = ((NM_LISTVIEW *)lParam)->iSubItem + IDM_SORTNAME;
            PostMessage (hWnd, WM_COMMAND, MAKELPARAM (i, 0), 0);
            break;

        // Ignore cache hinting for db example.
        case NM_DBLCLK:
            PostMessage (hWnd, WM_COMMAND, MAKELPARAM (IDM_EDIT, 0), 0);
            break;

        // Ignore cache hinting for db example.
        case LVN_ODCACHEHINT:
            break;

        case LVN_ODFINDITEM:
            // We should do a reverse look up here to see if
            // an item exists for the text passed.
            return -1;
        }
    }
    return 0;
}
//----------------------------------------------------------------------
// DoDestroyMain - Process WM_DESTROY message for window.
//
LRESULT DoDestroyMain (HWND hWnd, UINT wMsg, WPARAM wParam, 
                       LPARAM lParam) {
    PostQuitMessage (0);
    return 0;
}
//======================================================================
// Command handler routines
//----------------------------------------------------------------------
// DoMainCommandDelDB - Process Program Delete command.
//
LPARAM DoMainCommandDelDB (HWND hWnd, WORD idItem, HWND hwndCtl, 
                           WORD wNotifyCode) {
    int i, rc;

    i = MessageBox (hWnd, TEXT ("Delete the entire database?"), 
                    TEXT ("Delete"), MB_YESNO);
    if (i != IDYES) 
        return 0;
    if (g_oidDB) {
        CloseHandle (g_hDB);
        rc = CeDeleteDatabase (g_oidDB);
        if (rc == 0) {
            TCHAR szDbg[128];
            rc = GetLastError();
            wsprintf (szDbg, TEXT ("Couldn\'t delete db. rc=%d"), rc);
            MessageBox (hWnd, szDbg, szAppName, MB_OK);
            g_hDB = CeOpenDatabase (&g_oidDB, NULL, g_nLastSort, 
                                    0, hWnd);
            return 0;
        }
        g_hDB = 0;
        g_oidDB = 0;
    }
    ListView_SetItemCount (GetDlgItem (hWnd, ID_LISTV), 0);
    return 0;
}
//----------------------------------------------------------------------
// DoMainCommandExit - Process Program Exit command.
//
LPARAM DoMainCommandExit (HWND hWnd, WORD idItem, HWND hwndCtl, 
                          WORD wNotifyCode) {

    SendMessage (hWnd, WM_CLOSE, 0, 0);
    return 0;
}
//----------------------------------------------------------------------
// DoMainCommandNew - Process Program New command.
//
LPARAM DoMainCommandNew (HWND hWnd, WORD idItem, HWND hwndCtl, 
                         WORD wNotifyCode) {
    PCEPROPVAL pcepv;
    INT i, rc;
    CEOID oid;
    HWND hwndLV = GetDlgItem (hWnd, ID_LISTV);

    // Display the new/edit dialog.
    pcepv = 0;
    rc = DialogBoxParam (hInst, TEXT ("EditAlbumDlg"), hWnd, 
                         EditAlbumDlgProc, (LPARAM)&pcepv);
    if (rc == 0) 
        return 0;

    // Write the record.
    oid = CeWriteRecordProps(g_hDB, 0, NUM_DB_PROPS, pcepv);
    if (!oid) {
        TCHAR szText[64];
        rc = GetLastError ();
        wsprintf (szText, TEXT ("Write Rec fail. Error %d (%x)"),
                  rc, rc);
        MessageBox (hWnd, szText, TEXT ("Error"), MB_OK);
    }
    ClearCache ();                              // Clear the lv cache.

    i = ListView_GetItemCount (hwndLV) + 1;     // Increment list view
                                                // count.
    ListView_SetItemCount (hwndLV, i);
    InvalidateRect (hwndLV, NULL, TRUE);        // Force list view
                                                // redraw.
    return 0;
}
//----------------------------------------------------------------------
// DoMainCommandEdit - Process Program Edit command.
//
LPARAM DoMainCommandEdit (HWND hWnd, WORD idItem, HWND hwndCtl, 
                          WORD wNotifyCode) {
    PCEPROPVAL pcepv = 0;
    INT nSel, rc;
    WORD wProps = 0;
    DWORD dwRecSize, dwIndex;
    CEOID oid;
    HWND hwndLV = GetDlgItem (hWnd, ID_LISTV);

    nSel = ListView_GetSelectionMark (hwndLV);
    if (nSel == -1) 
        return 0;

    // Seek to the necessary record.
    oid = CeSeekDatabase (g_hDB, CEDB_SEEK_BEGINNING, nSel, &dwIndex);
    if (oid == 0) {
        TCHAR szTxt[64];
        INT rc = GetLastError();
        wsprintf (szTxt, TEXT ("Db item not found. rc = %d (%x)"),
                  rc, rc);
        MessageBox (NULL, szTxt, TEXT ("err"), MB_OK);
        return 0;
    }
    // Read all properties for the record.  Have the system
    // allocate the buffer containing the data.
    oid = CeReadRecordProps (g_hDB, CEDB_ALLOWREALLOC, &wProps, NULL,
                             &(LPBYTE)pcepv, &dwRecSize);
    if (oid == 0) {
        TCHAR szTxt[64];
        INT rc = GetLastError();
        wsprintf (szTxt, TEXT ("Db item not read. rc = %d (%x)"),
                  rc, rc);
        MessageBox (NULL, szTxt, TEXT ("err"), MB_OK);
        return 0;
    }
    // Display the edit dialog.
    rc = DialogBoxParam (hInst, TEXT ("EditAlbumDlg"), hWnd, 
                         EditAlbumDlgProc, (LPARAM)&pcepv);
    if (rc == 0) 
        return 0;

    // Write the record.
    oid = CeWriteRecordProps(g_hDB, oid, NUM_DB_PROPS, pcepv);
    if (!oid) {
        TCHAR szText[64];
        rc = GetLastError ();
        wsprintf (szText, TEXT ("Write Rec fail. Error %d (%x)"),
                   rc, rc);
        MessageBox (hWnd, szText, TEXT ("Error"), MB_OK);
    }
    LocalFree ((LPBYTE)pcepv);
    ClearCache ();                              // Clear the lv cache.

    InvalidateRect (hwndLV, NULL, TRUE);        // Force list view
                                                // redraw.
    return 0;
}
//----------------------------------------------------------------------
// DoMainCommandDelete - Process Program Delete command.
//
LPARAM DoMainCommandDelete (HWND hWnd, WORD idItem, HWND hwndCtl, 
                            WORD wNotifyCode) {
    HWND hwndLV;
    TCHAR szText[64];
    DWORD dwIndex;
    int i;
    CEOID oid;
    int nSel;

    hwndLV = GetDlgItem (hWnd, ID_LISTV);
    nSel = ListView_GetSelectionMark (hwndLV);
    if (nSel != -1) {

        wsprintf (szText, TEXT ("Delete this item?")); 
        i = MessageBox (hWnd, szText, TEXT ("Delete"), MB_YESNO);
        if (i != IDYES) 
            return 0;

        // Seek to the necessary record.
        oid = CeSeekDatabase (g_hDB, CEDB_SEEK_BEGINNING, nSel, &dwIndex);
        CeDeleteRecord (g_hDB, oid);

        // Reduce the list view count by one and force redraw.
        i = ListView_GetItemCount (hwndLV) - 1;
        ListView_SetItemCount (hwndLV, i);
        ClearCache ();                          // Clear the lv cache.
        InvalidateRect (hwndLV, NULL, TRUE);
    }
    return 0;
}
//----------------------------------------------------------------------
// DoMainCommandSort - Process the Sort commands.
//
LPARAM DoMainCommandSort(HWND hWnd, WORD idItem, HWND hwndCtl, 
                         WORD wNotifyCode) {
    int nSort;

    switch (idItem) {
    case IDM_SORTNAME:
        nSort = PID_NAME;
        break;
    case IDM_SORTARTIST:
        nSort = PID_ARTIST;
        break;
    case IDM_SORTCATEGORY:
        nSort = PID_CATEGORY;
        break;
    }
    if (nSort == g_nLastSort)
        return 0;

    ReopenDatabase (hWnd, nSort);      // Close and reopen the database.
    return 0;
}
//----------------------------------------------------------------------
// DoMainCommandAbout - Process the Help | About menu command.
//
LPARAM DoMainCommandAbout(HWND hWnd, WORD idItem, HWND hwndCtl, 
                          WORD wNotifyCode) {
    // Use DialogBox to create modal dialog.
    DialogBox (hInst, TEXT ("aboutbox"), hWnd, AboutDlgProc);
    return 0;
}
//----------------------------------------------------------------------
// CreateLV - Creates the list view control
//
HWND CreateLV (HWND hWnd, RECT *prect) {
    HWND hwndLV;
    LVCOLUMN lvc;

    // Create album list window.  
    hwndLV = CreateWindowEx (0, WC_LISTVIEW, TEXT (""), 
                         WS_VISIBLE | WS_CHILD | WS_VSCROLL |
                         LVS_OWNERDATA | WS_BORDER | LVS_REPORT, 
                         prect->left, prect->top,
                         prect->right - prect->left,
                         prect->bottom - prect->top,
                         hWnd, (HMENU)ID_LISTV, 
                         hInst, NULL);
    // Add columns.
    if (hwndLV) {
        lvc.mask = LVCF_TEXT | LVCF_WIDTH | LVCF_FMT | LVCF_SUBITEM;
        lvc.fmt = LVCFMT_LEFT;
        lvc.cx = 150;
        lvc.pszText = TEXT ("Name");
        lvc.iSubItem = 0;
        SendMessage (hwndLV, LVM_INSERTCOLUMN, 0, (LPARAM)&lvc);

        lvc.mask |= LVCF_SUBITEM;
        lvc.pszText = TEXT ("Artist");
        lvc.cx = 100;
        lvc.iSubItem = 1;
        SendMessage (hwndLV, LVM_INSERTCOLUMN, 1, (LPARAM)&lvc);

        lvc.mask |= LVCF_SUBITEM;
        lvc.pszText = TEXT ("Category");
        lvc.cx = 100;
        lvc.iSubItem = 2;
        SendMessage (hwndLV, LVM_INSERTCOLUMN, 2, (LPARAM)&lvc);
    }

    return hwndLV;
}
//----------------------------------------------------------------------
// OpenCreateDB - Open database, create if necessary.
//
HANDLE OpenCreateDB (HWND hWnd, int *pnRecords) {
    INT i, rc;
    CEOIDINFO oidinfo;
    SORTORDERSPEC sos[4];

    g_oidDB = 0;
    g_hDB = CeOpenDatabase (&g_oidDB, TEXT ("\\Albums"), 
                            g_nLastSort, 0, hWnd);
    if (g_hDB == INVALID_HANDLE_VALUE) {
        rc = GetLastError();
        if (rc == ERROR_FILE_NOT_FOUND) {
            i = 0;
            sos[i].propid = PID_NAME;
            sos[i++].dwFlags = 0;

            sos[i].propid = PID_ARTIST;
            sos[i++].dwFlags = 0;

            sos[i].propid = PID_CATEGORY;
            sos[i++].dwFlags = 0;

            g_oidDB = CeCreateDatabase (TEXT ("\\Albums"), 0, 3, 
                                        sos);
            if (g_oidDB == 0) {
                TCHAR szErr[128];
                wsprintf (szErr, TEXT ("Database create failed. \
                          rc %d"), GetLastError());
                MessageBox (hWnd, szErr, szAppName, MB_OK);
                return 0;
            }
            g_hDB = CeOpenDatabase(&g_oidDB,NULL, g_nLastSort, 0, hWnd);
        }
    } 
    CeOidGetInfo (g_oidDB, &oidinfo);
    *pnRecords = oidinfo.infDatabase.wNumRecords;
    return g_hDB;
}
//----------------------------------------------------------------------
// ClearCache - Clears the one item cache for the list view control.
//
void ClearCache (void) {

    if (g_pLastRecord)
        LocalFree (g_pLastRecord);
    g_pLastRecord = 0;            
    g_nLastItem = -1;
    return;
}
//----------------------------------------------------------------------
// ReopenDatabase - Closes and reopens the database 
//
void ReopenDatabase (HWND hWnd, INT nNewSort) {
    INT nCnt; 

    if (nNewSort != -1)
        g_nLastSort = nNewSort;

    if (g_hDB)
        CloseHandle (g_hDB);
    ClearCache ();                        // Clear the lv cache.

    g_hDB = OpenCreateDB (hWnd, &nCnt);

    ListView_SetItemCount (GetDlgItem (hWnd, ID_LISTV), nCnt);
    InvalidateRect (GetDlgItem (hWnd, ID_LISTV), NULL, 0);
    return;
}
//----------------------------------------------------------------------
// Get the album data from the database for the requested lv item.
//
int GetItemData (int nItem, PLVCACHEDATA pcd) {
    static WORD wProps;
    DWORD dwIndex;
    CEOID oid;
    PCEPROPVAL pRecord = NULL;
    DWORD dwRecSize;
    int i;

    // See if the item requested was the previous one.  If so,
    // just use the old data.
    if ((nItem == g_nLastItem) && (g_pLastRecord)) 
        pRecord = (PCEPROPVAL)g_pLastRecord;
    else {
        // Seek to the necessary record.
        oid = CeSeekDatabase (g_hDB, CEDB_SEEK_BEGINNING, nItem, &dwIndex);
        if (oid == 0) {
            TCHAR szTxt[64];
            INT rc = GetLastError();
            wsprintf (szTxt, TEXT ("Db item not found. rc = %d (%x)"),
                      rc, rc);
            MessageBox (NULL, szTxt, TEXT ("err"), MB_OK);
            return 0;
        }
        // Read all properties for the record.  Have the system
        // allocate the buffer containing the data.
        oid = CeReadRecordProps (g_hDB, CEDB_ALLOWREALLOC, &wProps, NULL,
                                 &(LPBYTE)pRecord, &dwRecSize);
        if (oid == 0) {
            TCHAR szTxt[64];
            INT rc = GetLastError();
            wsprintf (szTxt, TEXT ("Db item not read. rc = %d (%x)"),
                      rc, rc);
            MessageBox (NULL, szTxt, TEXT ("err"), MB_OK);
            return 0;
        }
        // Free old record and save the newly read one.
        if (g_pLastRecord)
            LocalFree (g_pLastRecord);
        g_nLastItem = nItem;
        g_pLastRecord = (LPBYTE)pRecord;

    }
    // Copy the data from the record to the album structure.
    for (i = 0; i < wProps; i++) {
        switch (pRecord->propid) {
        case PID_NAME:
            lstrcpy (pcd->Album.szName, pRecord->val.lpwstr);
            break;
        case PID_ARTIST:
            lstrcpy (pcd->Album.szArtist, pRecord->val.lpwstr);
            break;
        case PID_CATEGORY:
            pcd->Album.sCategory = pRecord->val.iVal;
            break;
        case PID_NUMTRACKS:
            pcd->Album.sNumTracks = pRecord->val.iVal;
            break;
        }
        pRecord++;
    }
    return 1;
}
//----------------------------------------------------------------------
// InsertLV - Add an item to the list view control.
//
INT InsertLV (HWND hWnd, INT nItem, LPTSTR pszName, LPTSTR pszType, 
              INT nSize) {
    LVITEM lvi;
    HWND hwndLV = GetDlgItem (hWnd, ID_LISTV);

    lvi.mask = LVIF_TEXT | LVIF_IMAGE | LVIF_PARAM;
    lvi.iItem = nItem;
    lvi.iSubItem = 0;
    lvi.pszText = pszName;
    lvi.iImage = 0;
    lvi.lParam = nItem;
    SendMessage (hwndLV, LVM_INSERTITEM, 0, (LPARAM)&lvi);

    lvi.mask = LVIF_TEXT;
    lvi.iItem = nItem;
    lvi.iSubItem = 1;
    lvi.pszText = pszType;
    SendMessage (hwndLV, LVM_SETITEM, 0, (LPARAM)&lvi);

    return 0;
}
//----------------------------------------------------------------------
// ValidateTime - Trival error checking of time field
//
BOOL ValidateTime (TCHAR *pStr) {
    BOOL fSep = FALSE;
    TCHAR *pPtr;

    pPtr = pStr;
    // See if field contains only numbers and up to one colon.
    while (*pPtr) {
        if (*pPtr == TEXT (':')) {
            if (fSep)
                return FALSE;
            fSep = TRUE;
        } else if ((*pPtr < TEXT ('0')) || (*pPtr > TEXT ('9'))) 
            return FALSE;
        pPtr++;
    }
    // Reject empty field.
    if (pPtr > pStr)
        return TRUE;
    return FALSE;
}
//======================================================================
// EditTrack dialog procedure
//
BOOL CALLBACK EditTrackDlgProc (HWND hWnd, UINT wMsg, WPARAM wParam, 
                                LPARAM lParam) {
    static LPTRACKINFO lpti;

    switch (wMsg) {
        case WM_INITDIALOG:
            lpti = (LPTRACKINFO)lParam;
            SendDlgItemMessage (hWnd, IDD_TRACK, EM_SETLIMITTEXT, 
                                sizeof (lpti->szTrack), 0);
            SendDlgItemMessage (hWnd, IDD_TIME, EM_SETLIMITTEXT, 
                                sizeof (lpti->szTime), 0);
            // See if new album or edit of old one.
            if (lstrlen (lpti->szTrack) == 0) {
                SetWindowText (hWnd, TEXT ("New Track"));
            } else {
                SetDlgItemText (hWnd, IDD_TRACK, lpti->szTrack);
                SetDlgItemText (hWnd, IDD_TIME, lpti->szTime);
            }
            return TRUE;

        case WM_COMMAND:
            switch (LOWORD (wParam)) {
                case IDOK:
                    Edit_GetText (GetDlgItem (hWnd, IDD_TRACK), 
                             lpti->szTrack, sizeof (lpti->szTrack));
                    Edit_GetText (GetDlgItem (hWnd, IDD_TIME), 
                              lpti->szTime, sizeof (lpti->szTime));
                    if (ValidateTime (lpti->szTime))
                        EndDialog (hWnd, 1);
                    else 
                        MessageBox (hWnd, TEXT ("Track time must \
be entered in mm:ss format"), 
                                    TEXT ("Error"), MB_OK);
                    return TRUE;
                case IDCANCEL:
                    EndDialog (hWnd, 0);
                    return TRUE;
            }
        break;
    }
    return FALSE;
}
//======================================================================
// EditAlbum dialog procedure
//
BOOL CALLBACK EditAlbumDlgProc (HWND hWnd, UINT wMsg, WPARAM wParam, 
                                LPARAM lParam) {
    static PCEPROPVAL *ppRecord;
    static int nTracks;
    PCEPROPVAL pRecord, pRecPtr;
    TCHAR *pPtr, szTmp[128];
    HWND hwndTList, hwndCombo;
    TRACKINFO ti;
    BOOL fEnable;
    INT i, nLen, rc;

    switch (wMsg) {
        case WM_INITDIALOG:
            ppRecord = (PCEPROPVAL *)lParam;
            pRecord = *ppRecord;

            hwndCombo = GetDlgItem (hWnd, IDD_CATEGORY);
            hwndTList = GetDlgItem (hWnd, IDD_TRACKS);

            Edit_LimitText (GetDlgItem (hWnd, IDD_NAME), MAX_NAMELEN);
            Edit_LimitText (GetDlgItem (hWnd, IDD_ARTIST),
                            MAX_ARTISTLEN);
            // Set tabstops on track list box.
            i = 110;
            ListBox_SetTabStops (hwndTList, 1, &i);
            // Initialize category combo box.
            for (i = 0; i < dim(pszCategories); i++) 
                ComboBox_AddString (hwndCombo, pszCategories[i]);
            ComboBox_SetCurSel (hwndCombo, 3);
            nTracks = 0;

            // See if new album or edit of old one.
            if (pRecord == 0) {
                SetWindowText (hWnd, TEXT ("New Album"));
            } else {
                // Copy the data from the record to album structure.
                for (i = 0; i < NUM_DB_PROPS; i++) {
                    switch (pRecord->propid) {
                    case PID_NAME:
                        SetDlgItemText (hWnd, IDD_NAME, 
                                        pRecord->val.lpwstr);
                        break;
                    case PID_ARTIST:
                        SetDlgItemText (hWnd, IDD_ARTIST, 
                                        pRecord->val.lpwstr);
                        break;
                    case PID_CATEGORY:
                        ComboBox_SetCurSel (hwndCombo, 
                                            pRecord->val.iVal);
                        break;
                    case PID_TRACKS:
                        pPtr = (TCHAR *)pRecord->val.blob.lpb;
                        for (i = 0; *pPtr; i++){
                          ListBox_InsertString (hwndTList,i,pPtr);
                            pPtr += lstrlen (pPtr) + 1;
                            nTracks++;
                        }
                        break;
                    }
                    pRecord++;
                }
            }
            // Select first track or disable buttons if no tracks.
            if (nTracks) 
                ListBox_SetCurSel (GetDlgItem (hWnd, IDD_TRACKS), 3);
            else {
                EnableWindow (GetDlgItem (hWnd, IDD_DELTRACK), 
                              FALSE);
                EnableWindow (GetDlgItem (hWnd, IDD_EDITTRACK), 
                              FALSE);
            }
            return TRUE;

        case WM_COMMAND:
            hwndTList = GetDlgItem (hWnd, IDD_TRACKS);
            hwndCombo = GetDlgItem (hWnd, IDD_CATEGORY);
            pRecord = *ppRecord;
            switch (LOWORD (wParam)) {
                case IDD_TRACKS:
                    switch (HIWORD (wParam)) {
                    case LBN_DBLCLK:
                        PostMessage (hWnd, WM_COMMAND, 
                                   MAKELONG(IDD_EDITTRACK, 0), 0);
                        break;
                    case LBN_SELCHANGE:
                        i = ListBox_GetCurSel (hwndTList);
                        if (i == LB_ERR) 
                            fEnable = FALSE;
                        else
                            fEnable = TRUE;
                        EnableWindow (GetDlgItem (hWnd, 
                                      IDD_DELTRACK), fEnable);
                        EnableWindow (GetDlgItem (hWnd, 
                                      IDD_EDITTRACK), fEnable);
                        break;
                    }
                    return TRUE;

                case IDD_NEWTRACK:
                    memset (&ti, 0, sizeof (ti));
                    rc = DialogBoxParam (hInst, 
                        TEXT ("EditTrackDlg"), hWnd, 
                        EditTrackDlgProc, (LPARAM)&ti);
                    if (rc) {
                        wsprintf (szTmp, TEXT ("%s\t%s"),
                                  ti.szTrack, ti.szTime);
                        i = ListBox_GetCurSel (hwndTList);
                        if (i != LB_ERR)
                            i++;
                        i = ListBox_InsertString (hwndTList, i,
                                                  szTmp);
                        ListBox_SetCurSel (hwndTList, i);
                    }
                    return TRUE;

                case IDD_EDITTRACK:
                    i = ListBox_GetCurSel (hwndTList);
                    if (i != LB_ERR) {
                        ListBox_GetText (hwndTList, i, szTmp);
                        pPtr = szTmp;
                        while ((*pPtr != TEXT ('\t')) &&
                               (*pPtr != TEXT ('\0'))) 
                            pPtr++;
                        if (*pPtr == TEXT ('\t')) 
                            *pPtr++ = TEXT ('\0');

                        lstrcpy (ti.szTime, pPtr);
                        lstrcpy (ti.szTrack, szTmp);
                        rc = DialogBoxParam (hInst, 
                                             TEXT ("EditTrackDlg"), 
                                             hWnd, EditTrackDlgProc, 
                                             (LPARAM)&ti);
                        if (rc) {
                            wsprintf (szTmp, TEXT ("%s\t%s"), 
                                      ti.szTrack, ti.szTime);
                            i = ListBox_GetCurSel (hwndTList);
                            ListBox_DeleteString (hwndTList, i);
                            ListBox_InsertString (hwndTList, i,
                                                  szTmp);
                            ListBox_SetCurSel (hwndTList, i);
                        }
                    }
                    return TRUE;

                case IDD_DELTRACK:
                    // Grab the current selection and remove
                    // it from list box.
                    i = ListBox_GetCurSel (hwndTList);
                    if (i != LB_ERR) {
                        rc = MessageBox (hWnd, 
                                         TEXT ("Delete this item?"),
                                         TEXT ("Track"), MB_YESNO);
                        if (rc == IDYES) {
                            i=ListBox_DeleteString (hwndTList,i);
                            if (i > 0)
                                i--;
                            ListBox_SetCurSel (hwndTList, i);
                        }
                    }
                    return TRUE;

                case IDOK:
                    // Be lazy and assume worst case size values.
                    nLen = sizeof (CEPROPVAL) * NUM_DB_PROPS + 
                           MAX_NAMELEN + MAX_ARTISTLEN + 
                           MAX_TRACKNAMELEN;
                    // See if prev record, alloc if not.
                    if (pRecord) {
                        // Resize record if necessary.
                        if (nLen > (int)LocalSize (pRecord)) 
                            pRecPtr = 
                               (PCEPROPVAL)LocalReAlloc (pRecord, 
                               nLen, LMEM_MOVEABLE);
                        else
                            pRecPtr = pRecord;
                    } else
                        pRecPtr = LocalAlloc (LMEM_FIXED, nLen);
                    if (!pRecPtr)
                        return 0;
                    // Copy the data from the controls to a 
                    // marshaled data block with the structure 
                    // at the front and  the data in the back.
                    pRecord = pRecPtr;
                    nTracks = ListBox_GetCount (hwndTList);
                    pPtr = (TCHAR *)((LPBYTE)pRecPtr + 
                             (sizeof (CEPROPVAL) * NUM_DB_PROPS));
                    // Zero structure to start over.
                    memset (pRecPtr, 0, LocalSize (pRecPtr));

                    pRecPtr->propid = PID_NAME;
                    pRecPtr->val.lpwstr = pPtr;
                    GetDlgItemText (hWnd, IDD_NAME, pPtr, 
                                    MAX_NAMELEN);
                    pPtr += lstrlen (pPtr) + 1;
                    pRecPtr++;

                    pRecPtr->propid = PID_ARTIST;
                    pRecPtr->val.lpwstr = pPtr;
                    GetDlgItemText (hWnd, IDD_ARTIST, pPtr, 
                                    MAX_ARTISTLEN);
                    pPtr += lstrlen (pPtr) + 1;
                    pRecPtr++;

                    pRecPtr->propid = PID_RELDATE;
                    pRecPtr->val.iVal = 0;
                    pRecPtr++;

                    pRecPtr->propid = PID_CATEGORY;
                    pRecPtr->val.iVal = 
                                    ComboBox_GetCurSel (hwndCombo);
                    pRecPtr++;

                    pRecPtr->propid = PID_NUMTRACKS;
                    pRecPtr->val.iVal = nTracks;
                    pRecPtr++;

                    pRecPtr->propid = PID_TRACKS;
                    pRecPtr->val.blob.lpb = (LPBYTE)pPtr;

                    // Get the track titles from the list box.
                    rc = MAX_TRACKNAMELEN;
                    for (i = 0; i < nTracks; i++) {
                        // Make sure we have the room in the buff.
                        rc -= ListBox_GetTextLen(hwndTList, i);
                        if (rc) 
                            ListBox_GetText (hwndTList, i, pPtr);
                        else {
                            nTracks = i;
                            break;
                        }
                        pPtr += lstrlen (pPtr) + 1;
                    }
                    *pPtr++ = TEXT ('\0');
                    pRecPtr->val.blob.dwCount = 
                              (LPBYTE)pPtr - pRecPtr->val.blob.lpb;
                    *ppRecord = pRecord;
                    EndDialog (hWnd, 1);
                    return TRUE;

                case IDCANCEL:
                    EndDialog (hWnd, 0);
                    return TRUE;
            }
        break;
    }
    return FALSE;
}
//======================================================================
// About dialog procedure
//
BOOL CALLBACK AboutDlgProc (HWND hWnd, UINT wMsg, WPARAM wParam, 
                            LPARAM lParam) {
    switch (wMsg) {
        case WM_COMMAND:
            switch (LOWORD (wParam)) {
                case IDOK:
                case IDCANCEL:
                    EndDialog (hWnd, 0);
                    return TRUE;
            }
        break;
    }
    return FALSE;
}
